import * as plugins from '../../../plugins.js';
import type { IAcmeOptions } from '../../../certificate/models/certificate-types.js';

/**
 * Configuration options for NetworkProxy
 */
export interface INetworkProxyOptions {
  port: number;
  maxConnections?: number;
  keepAliveTimeout?: number;
  headersTimeout?: number;
  logLevel?: 'error' | 'warn' | 'info' | 'debug';
  cors?: {
    allowOrigin?: string;
    allowMethods?: string;
    allowHeaders?: string;
    maxAge?: number;
  };

  // Settings for SmartProxy integration
  connectionPoolSize?: number; // Maximum connections to maintain in the pool to each backend
  portProxyIntegration?: boolean; // Flag to indicate this proxy is used by SmartProxy
  useExternalPort80Handler?: boolean; // Flag to indicate using external Port80Handler
  // Protocol to use when proxying to backends: HTTP/1.x or HTTP/2
  backendProtocol?: 'http1' | 'http2';

  // ACME certificate management options
  acme?: IAcmeOptions;
}

/**
 * Interface for a certificate entry in the cache
 */
export interface ICertificateEntry {
  key: string;
  cert: string;
  expires?: Date;
}

/**
 * Interface for reverse proxy configuration
 */
export interface IReverseProxyConfig {
  destinationIps: string[];
  destinationPorts: number[];
  hostName: string;
  privateKey: string;
  publicKey: string;
  authentication?: {
    type: 'Basic';
    user: string;
    pass: string;
  };
  rewriteHostHeader?: boolean;
  /**
   * Protocol to use when proxying to this backend: 'http1' or 'http2'.
   * Overrides the global backendProtocol option if set.
   */
  backendProtocol?: 'http1' | 'http2';
}

/**
 * Interface for connection tracking in the pool
 */
export interface IConnectionEntry {
  socket: plugins.net.Socket;
  lastUsed: number;
  isIdle: boolean;
}

/**
 * WebSocket with heartbeat interface
 */
export interface IWebSocketWithHeartbeat extends plugins.wsDefault {
  lastPong: number;
  isAlive: boolean;
}

/**
 * Logger interface for consistent logging across components
 */
export interface ILogger {
  debug(message: string, data?: any): void;
  info(message: string, data?: any): void;
  warn(message: string, data?: any): void;
  error(message: string, data?: any): void;
}

/**
 * Creates a logger based on the specified log level
 */
export function createLogger(logLevel: string = 'info'): ILogger {
  const logLevels = {
    error: 0,
    warn: 1,
    info: 2,
    debug: 3
  };

  return {
    debug: (message: string, data?: any) => {
      if (logLevels[logLevel] >= logLevels.debug) {
        console.log(`[DEBUG] ${message}`, data || '');
      }
    },
    info: (message: string, data?: any) => {
      if (logLevels[logLevel] >= logLevels.info) {
        console.log(`[INFO] ${message}`, data || '');
      }
    },
    warn: (message: string, data?: any) => {
      if (logLevels[logLevel] >= logLevels.warn) {
        console.warn(`[WARN] ${message}`, data || '');
      }
    },
    error: (message: string, data?: any) => {
      if (logLevels[logLevel] >= logLevels.error) {
        console.error(`[ERROR] ${message}`, data || '');
      }
    }
  };
}